/* toc.css - enable table of contents */

.man-navigation {
    display:block !important;
    position:fixed;
    top:0;
    left:113ex; /* .mp width + padding */
    height:100%;
    width:100%;
    padding:48px 0 0 0;
    border-left:1px solid #dbdbdb;
    background:#eee;
}
.man-navigation a,
.man-navigation a:hover,
.man-navigation a:link,
.man-navigation a:visited {
    display:block;
    margin:0;
    padding:5px 2px 5px 30px;
    color:#999;
    text-decoration:none;
}
.man-navigation a:hover {
    color:#111;
    text-decoration:underline;
}
